package org.processmining.acceptingpetrinetredundantplacesreductor.plugins;

import java.util.Collection;

import org.processmining.acceptingpetrinet.models.AcceptingPetriNet;
import org.processmining.acceptingpetrinetredundantplacesreductor.algorithms.RemoveStructuralRedundantPlacesAlgorithm;
import org.processmining.acceptingpetrinetredundantplacesreductor.connections.RemoveStructuralRedundantPlacesConnection;
import org.processmining.acceptingpetrinetredundantplacesreductor.help.RemoveStructuralRedundantPlacesHelp;
import org.processmining.acceptingpetrinetredundantplacesreductor.parameters.RemoveStructuralRedundantPlacesParameters;
import org.processmining.contexts.uitopia.annotations.UITopiaVariant;
import org.processmining.framework.connections.ConnectionCannotBeObtained;
import org.processmining.framework.plugin.PluginContext;
import org.processmining.framework.plugin.annotations.Plugin;
import org.processmining.framework.plugin.annotations.PluginVariant;
import org.processmining.models.connections.petrinets.behavioral.InitialMarkingConnection;
import org.processmining.models.semantics.petrinet.Marking;

@Plugin(name = "Remove Structural Redundant Places from Accepting Petri Net", parameterLabels = {
		"Accepting Petri Net", "Parameters" }, returnLabels = { "Accepting Petri Net" }, returnTypes = { AcceptingPetriNet.class }, help = RemoveStructuralRedundantPlacesHelp.TEXT)
public class RemoveStructuralRedundantPlacesPlugin extends RemoveStructuralRedundantPlacesAlgorithm {

	@UITopiaVariant(affiliation = UITopiaVariant.EHV, author = "H.M.W. Verbeek", email = "h.m.w.verbeek@tue.nl")
	@PluginVariant(variantLabel = "Remove Structural Redundant Places, Default", requiredParameterLabels = { 0 })
	public AcceptingPetriNet runDefault(PluginContext context, AcceptingPetriNet net) {
		RemoveStructuralRedundantPlacesParameters parameters = new RemoveStructuralRedundantPlacesParameters();
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use runDefault() instead.
	 */
	@Deprecated
	public AcceptingPetriNet reduceDefault(PluginContext context, AcceptingPetriNet net) {
		return runDefault(context, net);
	}

	@PluginVariant(variantLabel = "Remove Structural Redundant Places, Parameters", requiredParameterLabels = { 0, 1 })
	public AcceptingPetriNet run(PluginContext context, AcceptingPetriNet net,
			RemoveStructuralRedundantPlacesParameters parameters) {
		return runConnection(context, net, parameters);
	}

	/**
	 * @deprecated Use run() instead.
	 */
	@Deprecated
	public AcceptingPetriNet reduceParameters(PluginContext context, AcceptingPetriNet net,
			RemoveStructuralRedundantPlacesParameters parameters) {
		return run(context, net, parameters);
	}

	private AcceptingPetriNet runConnection(PluginContext context, AcceptingPetriNet net,
			RemoveStructuralRedundantPlacesParameters parameters) {
		if (parameters.isTryConnections()) {
			Collection<RemoveStructuralRedundantPlacesConnection> connections;
			if (context != null) {
				try {
					connections = context.getConnectionManager().getConnections(
							RemoveStructuralRedundantPlacesConnection.class, context, net);
					for (RemoveStructuralRedundantPlacesConnection connection : connections) {
						if (connection.getObjectWithRole(RemoveStructuralRedundantPlacesConnection.NET).equals(net)
								&& connection.getParameters().equals(parameters)) {
							return connection.getObjectWithRole(RemoveStructuralRedundantPlacesConnection.REDUCEDNET);
						}
					}
				} catch (ConnectionCannotBeObtained e) {
				}
			}
		}
		AcceptingPetriNet reducedNet = apply(context, net, parameters);
		if (parameters.isTryConnections()) {
			if (context != null) {
				context.getProvidedObjectManager().createProvidedObject(
						"Initial marking for " + reducedNet.getNet().getLabel(), reducedNet.getInitialMarking(),
						Marking.class, context);
				context.addConnection(new InitialMarkingConnection(reducedNet.getNet(), reducedNet.getInitialMarking()));
				context.getConnectionManager().addConnection(
						new RemoveStructuralRedundantPlacesConnection(net, reducedNet, parameters));
			}
		}
		return reducedNet;
	}

}